##########################################################################
#check if peak is a rotamer of primary peak

def is_rotamer(primary, peak):
	
	flag = False
	boundaries = []
	for i in range(3):
		boundaries.append([0, 0])
	
	#collect bracketed rotamer values
	pad = 30
	boundaries[0][0] = primary + pad
	boundaries[0][1] = primary - pad
	boundaries[1][0] = primary + 120 + pad
	boundaries[1][1] = primary + 120 - pad
	boundaries[2][0] = primary - 120 + pad
	boundaries[2][1] = primary - 120 - pad
	
	#shift into proper degree range
	for xyz in boundaries:
		for high_low in xyz:
			if high_low > 350:
				high_low -= 360
			if high_low < 0:
				high_low += 360
	
	for xyz in boundaries:
		#if in standard rotamer range
		if peak <= xyz[0] and peak >= xyz[1]:
			flag = True
		#deal with edge effects
		if xyz[0] < xyz[1]:
			if peak <= xyz[0]:
				flag = True
			if peak >= xyz[1]:
				flag = True
	return flag
				
##########################################################################
#check if peak is a rotamer of primary peak
def collect_peaks_byrot(peak_list):

	#create containers for final data collection
	num_peaks = []
	num_rot = []
	for pClass in peak_list:
		tmp_peaks = []
		tmp_rot = []
		for peak in range(7):
			tmp_peaks.append(0.0)
			tmp_rot.append(0.0)
		num_peaks.append(tmp_peaks)
		num_rot.append(tmp_rot)
	
	for i, pClass in enumerate(peak_list):
		for peak in pClass[1]: 
			if len(peak) > 0:
				
				#collect overall data
				if pClass[0] == 'Ala' or pClass[0] == 'Methyl':
					primaryRot = 60
				else:
					primaryRot = peak[0][1]
				for j, x in enumerate(peak[:7]): #stop collecting after seventh peak
					num_peaks[i][j] += 1
					if is_rotamer(primaryRot, peak[j][1]):
						num_rot[i][j] = num_rot[i][j] + 1
				
	return num_peaks, num_rot   
	 
##########################################################################
# prints data out to output file 

def calc_percent(data, pSum): 

	for dInd, d in enumerate(data):
		if d > 0:
			data[dInd] /= pSum
			data[dInd] *= 100
	return data

##########################################################################
# write out statistics for rotamers for test set

def write_rot_stats(parms, peakTot, rotTot, peaks_byclass):

	#print overall percentages
	parms.outfile.write("   .............Rotameric Percentages.............\n")
	parms.outfile.write("   Percent of total residues that contain each\n")
	parms.outfile.write("   type of peak.\n\n")
	
	labels = ['Primary', 'Secondary', 'Tertiary', 'Quarternary', 'Fifth', 'Sixth', 'Seventh']
	
	for pClass, pTot, rTot in zip(peaks_byclass, peakTot, rotTot):
		parms.outfile.write('      %-8s:%17s%19s\n' 
					%(pClass[0], 'Total Percents', 'Rotamer Percents'))
	
		pTot = calc_percent(pTot, len(pClass[1]))
		rTot = calc_percent(rTot, len(pClass[1]))
		for l, p, r in zip(labels, pTot, rTot):
			parms.outfile.write('         %-11s%8.1f%%%16.1f%%\n' %(l, p, r))
		parms.outfile.write('\n')
	
	
