
##########################################################################
#pick peaks for each residue	
def peak_picking(parms, res, index=0):

	lower = parms.lowerCutoff
	if index > 0:
		parms.upperCutoff = False
		if parms.mapType == 'sigma':
			lower = -999
	
	if parms.noisemapfileName:
		if res.chi1 or res.ring:
			res.peakList[index].peak1 = dynamic_cut_peaks(res.peakList[index].sigma1,
							res.noiseList[index].sigma1, lower)
		if res.chi2:
			res.peakList[index].peak2 = dynamic_cut_peaks(res.peakList[index].sigma2,
							res.noiseList[index].sigma2, lower)
		if res.chi3:
			res.peakList[index].peak3 = dynamic_cut_peaks(res.peakList[index].sigma3,
							res.noiseList[index].sigma3, lower)
		if res.chi4:
			res.peakList[index].peak4 = dynamic_cut_peaks(res.peakList[index].sigma4,
							res.noiseList[index].sigma4, lower)
		
	else:
		if res.chi1 or res.ring:
			res.peakList[index].peak1 = hard_cut_peaks(res.peakList[index].sigma1, lower)			   	
		if res.chi2:			
			res.peakList[index].peak2 = hard_cut_peaks(res.peakList[index].sigma2, lower)
		if res.chi3:
			res.peakList[index].peak3 = hard_cut_peaks(res.peakList[index].sigma3, lower)
		if res.chi4:
			res.peakList[index].peak4 = hard_cut_peaks(res.peakList[index].sigma4, lower)
	
	if parms.upperCutoff:
		res.peakList[index].peak1 = upper_cut(res.peakList[index].peak1, 
							parms.upperCutoffValue)
		res.peakList[index].peak2 = upper_cut(res.peakList[index].peak2, 
							parms.upperCutoffValue) 
		res.peakList[index].peak3 = upper_cut(res.peakList[index].peak3, 
							parms.upperCutoffValue) 
		res.peakList[index].peak4 = upper_cut(res.peakList[index].peak4, 
							parms.upperCutoffValue)
	
	
##########################################################################
#HARD LOWER CUTOFF########################################################				
##########################################################################
#loop over sigma values and collect peaks
def hard_cut_peaks(sigma_list, lower):

	peak_list = []
	for i, den in enumerate(sigma_list):
		#collect electron density for peaks
		#check if sigma fall above lower cutoff					
		#if sigma is within bounds, check if it is a peak
		if den[0] > lower:
			if i == 0:
				flag = is_peak(sigma_list[i][0], sigma_list[-1][0], sigma_list[i+1][0])
			elif i == (len(sigma_list) - 1):
				flag = is_peak(sigma_list[i][0], sigma_list[i-1][0], sigma_list[0][0])
			else:
				flag = is_peak(sigma_list[i][0], sigma_list[i-1][0], sigma_list[i+1][0])
			if flag == True:
				peak_list.append(den)
			
	peak_list.sort() #NOTE:  sorts in ascending order
	peak_list.reverse()
	
	dupes = []
	for a in peak_list:
		for b in peak_list:
			if abs(a[1]-b[1]) == 1:
				dupes.append(a)
	if len(dupes) > 0:
		from math import floor
		index = int(floor(len(dupes)/2)) - 1
		badPeaks = []
		for xInd, x in enumerate(dupes):
			if xInd != index:
				badPeaks.append(peak_list.index(x))
		tmpPeakList = []
		for pInd,p in enumerate(peak_list):
			if pInd in badPeaks:
				continue
			else:
				tmpPeakList.append(p)
		return tmpPeakList
	else:			
		return peak_list
			
##########################################################################
#define if sigma value is peak
def is_peak(curr, before, after):

	return curr >= before and curr >= after
		
##########################################################################
#remove all non-primary peaks above upper cutoff
def upper_cut(peaks, upper):
	
	tmp_peak =[]
	for x, den in enumerate(peaks):
		cut = False
		if x != 0:
			if den[0] > upper:
				cut = True
		if cut == False:
			tmp_peak.append(den)
	return tmp_peak
	
##########################################################################	
#loop over sigma values and collect peaks
def dynamic_cut_peaks(sigma_list, noise, lower):

	peak_list = []
	for i, den in enumerate(sigma_list):
		#collect electron density for peaks
		#check if sigma fall above noise level					
		#if sigma is within bounds, check if it is a peak
		if (den[0] > noise[i][0]) and (den[0] > lower):
			if i == 0:
				flag = is_peak(sigma_list[i][0], sigma_list[-1][0], sigma_list[i+1][0])
			elif i == (len(sigma_list) - 1):
				flag = is_peak(sigma_list[i][0], sigma_list[i-1][0], sigma_list[0][0])
			else:
				flag = is_peak(sigma_list[i][0], sigma_list[i-1][0], sigma_list[i+1][0])
			if flag == True:
				peak_list.append(den)
			
	peak_list.sort() #NOTE:  sorts in ascending order
	peak_list.reverse()

	dupes = []
	for a in peak_list:
		for b in peak_list:
			if abs(a[1]-b[1]) == 1:
				dupes.append(a)
	if len(dupes) > 0:
		from math import floor
		index = int(floor(len(dupes)/2)) - 1
		badPeaks = []
		for xInd, x in enumerate(dupes):
			if xInd != index:
				badPeaks.append(peak_list.index(x))
		tmpPeakList = []
		for pInd,p in enumerate(peak_list):
			if pInd in badPeaks:
				continue
			else:
				tmpPeakList.append(p)
		return tmpPeakList
	else:			
		return peak_list
