import sys
############################################################################
#  calculates distance between two atoms

def dist(a1, a2):

	dist = 0
	for x1, x2 in zip(a1, a2):
		dist += pow((x2 - x1),2)
	import math
	dist = math.sqrt(dist)
	
	return dist
	
############################################################################
#  generates rings at appropriate chi angles

def gen_rings(parms, chiList):

 	from chimera import molEdit
    	chi_list = []
	ang_list = []
	vec_list = []
	#generate rings of coordinates
	for chi in chiList:
		
		tmp_chi = []
		tmp_ang = []
		tmp_vec = []
		a0 = chi[0].coord
		a1 = chi[1].coord
		a2 = chi[2].coord

		#default sampling values for torsion drive
		angle = 111.10
		bond = 1.53

		#if chi angle ends with a heavy atom
		if len(chi) > 3:
		       
		       #if dynamic standard bond angles and lengths 
		       #are requested
		       if parms.atomSample == 'dynamic':
				
				#get bond lengths and angles from 
				#from parameter file
				try:
					values = parms.AMBERtype['%s:%s:%s' 
							%(chi[1].type, 
							chi[2].type, 
							chi[3].type)]
					bond = values[0]
					angle = values[1]
				except:
					print "WARNING:  Atom combination %s:%s:%s" %(chi[1].type,
											chi[2].type,
											chi[3].type)
					print "not found!",
					print "Using default bond length and angles ",
					print "for sampling."
					
					
                       #if experimental bond angles and distances
		       #are requested
		       if parms.atomSample == 'experimental':  
		
		       		#calculate bond length from experimental atoms
                       		bond = dist(a2, chi[3].coord)
                                              
                       		#calculate bond angle from experimental atoms
                       		import chimera
                       		angle = chimera.angle(a1, a2, chi[3].coord)

		#generate ring of atoms a specific sampling interval
		dihed = 0
		while dihed < 360:
			#create new atom at new dihedrral
			coord = molEdit.findPt(a2, a1, a0, 
				               bond, angle, dihed)
			tmp_chi.append(coord)
			tmp_ang.append(dihed)
				
			dihed += parms.chiSample
			
		chi_list.append(tmp_chi)
		ang_list.append(tmp_ang)
	
	return chi_list, ang_list
    
############################################################################
#  determine densities for rings

def ring_densities(rings, angle, dens):    

	import pdbResidues
	sigma = pdbResidues.Sigma()
	
	try: tmp_dens = dens.interpolated_values(rings[0])
	except IndexError: return sigma
	for i, d in enumerate(tmp_dens):
		sigma.sigma1.append([d, angle[0][i]])
	
	try: tmp_dens = dens.interpolated_values(rings[1])
	except IndexError: return sigma
	for i, d in enumerate(tmp_dens):
		sigma.sigma2.append([d, angle[1][i]])

	try: tmp_dens = dens.interpolated_values(rings[2])
	except IndexError: return sigma
	for i, d in enumerate(tmp_dens):
		sigma.sigma3.append([d, angle[2][i]])
		
	try: tmp_dens = dens.interpolated_values(rings[3])
	except IndexError: return sigma
	for i, d in enumerate(tmp_dens):
		sigma.sigma4.append([d, angle[3][i]])

	return sigma
	
