import sys

##########################################################################
# write out sigma values for plotting
def write_sigmaplot(parms, residues, TYPE):

	parms.outfile.write('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
	parms.outfile.write('~~~~~~~~~~~~~~~~~\n')
	parms.outfile.write('Writing out data for plots...\n\n')
	
	title = ''
	plot_sigma = []
	
	for r in residues:
	    try:
	    	if len(r.peakList[0].sigma1) > 0:
	    	    title = title + '\t' + r.fullName + '_' + 'chi1'
	    	    if TYPE == 'signal':
			plot_sigma.append(r.peakList[0].sigma1)
		    if TYPE == 'noise':
			plot_sigma.append(r.noiseList[0].sigma1)				
	    	if len(r.peakList[0].sigma2) > 0:
	    	    title = title + '\t' + r.fullName + '_' + 'chi2'
		    if TYPE == 'signal':
	    		plot_sigma.append(r.peakList[0].sigma2)
		    if TYPE == 'noise':
	    		plot_sigma.append(r.noiseList[0].sigma2)
	    	if len(r.peakList[0].sigma3) > 0:
	    	    title = title + '\t' + r.fullName + '_' + 'chi3'
		    if TYPE == 'signal':
	    		plot_sigma.append(r.peakList[0].sigma3)
		    if TYPE == 'noise':
			plot_sigma.append(r.noiseList[0].sigma3)
	    	if len(r.peakList[0].sigma4) >0:
	    	    title = title + '\t' + r.fullName + '_' + 'chi4'
		    if TYPE == 'signal':
			    plot_sigma.append(r.peakList[0].sigma4)
		    if TYPE == 'noise':
			    plot_sigma.append(r.noiseList[0].sigma4)
	    except IndexError or TypeError:
	    	parms.outfile.write("	ERROR: Residue %s" %r.id)
	    	parms.outfile.write("has no sigma values!\n")
	
	fileTitle = TYPE
	if len(plot_sigma) > 0:
	    out = open('%s.%s_plot.txt' %(parms.verbOutfileName,fileTitle), 'w')
	    out.write('%s\n' %title)
	    size = 0
	    for d in plot_sigma:
	    	if len(d) > size:
			size = len(d)
	    for d in range(size):
	    	data = ''
		for res in plot_sigma:
		    data += '%i %.3f\t' %(res[d][1], res[d][0])
		out.write('%s\n' %data)
	    out.close()
	else:
	    parms.outfile.write('   WARNING:  No amino acids found for ')
	    parms.outfile.write('plotting\n.')

##########################################################################
#write out verbose list of peaks	  
def write_peaklist(parms, peaks_byclass):
	
	#set up files for writing
	ala_out = open('%s.ala_peaklist.txt' %parms.verbOutfileName, 'w')
	ala_out.write('Residue PrimarySigmaValue PrimaryChiAngle; [SecondarySigmaValue SecondaryChiAngle; etc]\n')
	chi1_out = open('%s.ubchi1_peaklist.txt' %parms.verbOutfileName, 'w')
	chi1_out.write('Residue PrimarySigmaValue PrimaryChiAngle; [SecondarySigmaValue SecondaryChiAngle; etc]\n')
	ring_out = open('%s.ring_peaklist.txt' %parms.verbOutfileName, 'w')
	ring_out.write('Residue PrimarySigmaValue PrimaryChiAngle; [SecondarySigmaValue SecondaryChiAngle; etc]\n')
	chi2_out = open('%s.ubchi2_peaklist.txt' %parms.verbOutfileName, 'w')
	chi2_out.write('Residue PrimarySigmaValue PrimaryChiAngle; [SecondarySigmaValue SecondaryChiAngle; etc]\n')
	chi3_out = open('%s.chi3_peaklist.txt' %parms.verbOutfileName, 'w')
	chi3_out.write('Residue PrimarySigmaValue PrimaryChiAngle; [SecondarySigmaValue SecondaryChiAngle; etc]\n')
	chi4_out = open('%s.chi4_peaklist.txt' %parms.verbOutfileName, 'w')
	chi4_out.write('Residue PrimarySigmaValue PrimaryChiAngle; [SecondarySigmaValue SecondaryChiAngle; etc]\n')
	
	for c in peaks_byclass:
		if c[0] == 'Ala':
			tot_ala = 0
			for i, res in enumerate(c[1]):
				if len(res) > 0:
					pList = ''
					for p in res:
						pList += '%.3f %s; ' %(p[0], p[1])
					ala_out.write('%s\t%s\n' %(c[2][i], pList))
					tot_ala += 1
			ala_out.write('SUMMARY:  Alanine Peaks=%s Total Alanines=%s'
			%(tot_ala, len(c[1])))
		elif c[0] == 'UB Chi 1':
			for i, res in enumerate(c[1]):
				if len(res) > 0:
					pList = ''
					for p in res:
						pList += '%.3f %s; ' %(p[0], p[1])
					chi1_out.write('%s\t%s\n' %(c[2][i], pList))
		elif c[0] == 'Ring':
			for i, res in enumerate(c[1]):
				if len(res) > 0:
					pList = ''
					for p in res:
						pList += '%.3f %s; ' %(p[0], p[1])
					ring_out.write('%s\t%s\n' %(c[2][i], pList))
		elif c[0] == 'UB Chi 2':
			for i, res in enumerate(c[1]):
				if len(res) > 0:
					pList = ''
					for p in res:
						pList += '%.3f %s; ' %(p[0], p[1])
					chi2_out.write('%s\t%s\n' %(c[2][i], pList))
		elif c[0] == 'Chi 3':
			for i, res in enumerate(c[1]):
				if len(res) > 0:
					pList = ''
					for p in res:
						pList += '%.3f %s; ' %(p[0], p[1])
					chi3_out.write('%s\t%s\n' %(c[2][i], pList))
		elif c[0] == 'Chi 4':
			for i, res in enumerate(c[1]):
				if len(res) > 0:
					pList = ''
					for p in res:
						pList += '%.3f %s; ' %(p[0], p[1])
					chi4_out.write('%s\t%s\n' %(c[2][i], pList))

	#clean up files			
	ala_out.close()
	chi1_out.close()
	chi2_out.close()
	chi3_out.close()
	chi4_out.close()

############################################################################
#  prints data to for chi2/chi1 correlation plot

def write_chi2chi1(parms, residues):

	out = open('%s.chi2chi1_peaklist.txt' %parms.verbOutfileName, 'w')
	out.write('PDB\tResidue\t2nd Chi 1\t1st Chi 2\n')
	
	for r in residues:
		if r.name != 'ALA' and len(r.peakList) > 1: 
			out.write('%s\t%s\t' %(r.pdbCode,r.fullName))
			out.write('%s\t' %r.peakList[0].peak1[1][1])
			if len(r.peakList[1].peak1) > 0:
				out.write('%s' %r.peakList[1].peak1[0][1])
			out.write('\n')

############################################################################
#writes out rings in pdb format
def write_rings(res, rings, index):

	out = open('%s_ring%s.pdb' %(res.fullName, index), 'w')
	
	for xInd,x in enumerate(rings[0]):
		name = res.fullName.strip().split('_')
		out.write('ATOM%7i   O  TMP A%4s' %(xInd+1,name[-1]))
		out.write('%12.3f%8.3f%8.3f  1.00 20.00\n' %(x[0], x[1], x[2]))
	out.close()
	
